% This program calculates the energy transport of a dipole in a multilayer stack
% Coded and developed at MIT by Kemal Celebi and Marc Baldo [1]
% Particular figures this code generates (for a dipole in the Alq3 layer):
% 1. Total outcoupling and energy lost or guided in other layers of the stack
% 2. Angular distribution of a perpendicular dipole
% 3. Angular distribution of a parallel dipole 
% 4. Mode distribution of a perpendicular dipole
% 5. Mode distribution of a parallel dipole
% The OLED structure used in this code is the device by Segal et al [2]
% References:
% [1] K. Celebi, T. D. Heidel, and M. A. Baldo, "Simplified calculation of dipole energy transport in a multilayer stack using dyadic Greens functions," Opt. Express 15, 1762-1772 (2007)
% [2] M. Segal, M. A. Baldo, R. J. Holmes, S. R. Forrest and Z. G. Soos, "Excitonic singlet-triplet ratios in molecular and polymeric organic materials," Phys. Rev. B 68, 075211 (2003).


clf, clear

% ------------------------------ CONSTANTS --------------------------------
Planck=6.62e-34; charge=1.6e-19; light=3e8;
q=1; %radiative efficiency of the dipole
t0=1e-9; %free space lifetime (assumed 1ns) 
% some constants not used in this code
% -------------------------------------------------------------------------


% ---------------------- Dielectric constant data -------------------------
%First row is wavelength data (in nm) second row is dielectric constants
air = [200 1920; 1 1];
ITO = [200 250 300 350 400 450 500 550 600 650 700 750 800 850 900; 2.6000+i*2.8800 4.3500+i*3.0800 5.6700+i*1.4400 4.6176+i*0.3010 3.9996+i*0.0800 3.6099+i*0.0380 3.4224+i*0.0370 3.2399+i*0.0360 3.1683+i*0.0356 3.0624+i*0.0350 2.9928+i*0.0346 2.8899+i*0.0340 2.8222+i*0.0504 2.7554+i*0.0498 2.6567+i*0.0489];
gold = [317.897 326.263 335.081 344.389 354.229 364.647 375.697 387.438 399.935 413.267 427.517 442.786 459.185 476.846 495.92 516.583 539.044 563.545 652.526 688.778 729.294 774.875 826.533 885.571 953.692 1033.17; -0.2396 + 7.0067i  -0.1999 + 6.8509i  -0.2268 + 6.6886i  -0.2890 + 6.5201i  -0.3875 + 6.4310i  -0.5224 + 6.3904i  -0.7564 + 6.4652i  -0.9458 + 6.4817i  -1.0770 + 6.4861i  -1.1573 + 6.4066i  -1.1521 + 6.2701i  -1.1854 + 5.9481i  -1.3742 + 5.2648i  -1.6831 + 4.4613i  -2.5465 + 3.3709i  -4.1247 + 2.5779i  -6.2900 + 2.0422i  -8.2008 + 1.7626i  -9.8949 + 1.0458i -14.4144 + 1.2160i -18.8956 + 1.4268i -23.5893 + 1.6913i -29.0168 + 2.0266i -34.5303 + 2.4696i -41.8052 + 3.0538i -49.9109 + 3.8461i];
ag_nk=load('ag.nk'); ag = [ag_nk(:,1)'; ag_nk(:,2)'.^2-ag_nk(:,3)'.^2+2*i*ag_nk(:,2)'.*ag_nk(:,3)'];
mg_nk=load('Mg_palik.nk'); mg = [mg_nk(:,1)'; mg_nk(:,2)'.^2-mg_nk(:,3)'.^2+2*i*mg_nk(:,2)'.*mg_nk(:,3)'];
bcp_nk=load('bcp.nk'); bcp = [bcp_nk(:,1)'; bcp_nk(:,2)'.^2-bcp_nk(:,3)'.^2+2*i*bcp_nk(:,2)'.*bcp_nk(:,3)'];
pedot_nk=load('pedot.nk'); pedot = [pedot_nk(:,1)'; pedot_nk(:,2)'.^2-pedot_nk(:,3)'.^2+2*i*pedot_nk(:,2)'.*pedot_nk(:,3)'];
alq3_nk =dlmread('alq3_literature.dat',',',0,0); alq3 = [0.1*alq3_nk(:,1)'; alq3_nk(:,2)'.^2-alq3_nk(:,3)'.^2+2*i*alq3_nk(:,2)'.*alq3_nk(:,3)'];
test_glass=[200 1920; (1.5+i*0.0001)^2 (1.5+i*0.0001)^2];
glass_no_loss=[200 1920; 1.5^2 1.5^2];
tpd=bcp;
% -------------------------------------------------------------------------

% ---------------------- Define and discretize u --------------------------
x_res=5e-4;
x_real=[-pi/2:x_res:-x_res]; % this is for the u=0 to u=1 region, it is negative so that u is continuous from x_real to x_imag
x_imag=i*[x_res:x_res:1.8]; % this is the u>1 region, change the upper bound to get more accurate calculations of non-rad energy transfer very close to a metal
% If the upper bound is too large, calculation diverges
dx=[diff([x_real 0]) -diff([0 x_imag])];
x=[x_real x_imag]; u=cos(x); % u is the normalized surface-parallel wavevector
size_u=max(size(u)); u_eq_1=max(size(x_real));
du=[diff(cos([x_real 0])) diff(cos([0 x_imag]))];
% -------------------------------------------------------------------------    

% ------------------------ Define the structure ---------------------------
thickness_array=[0:100e-10:0.45e-6];
thickness=[200e-10 1000e-10 500e-10 200e-10 500e-10 300e-10 1600e-10 5000e-10]; % 1600 is ITO, from Segal et al PRB
dipole_position_array=1e-10*[0:20:200];
dipole_layer=5;
wavelength=535; %in nanometers
epsilon(10)=interp1(glass_no_loss(1,:),glass_no_loss(2,:),wavelength);
epsilon(9)=interp1(glass_no_loss(1,:),glass_no_loss(2,:),wavelength);
epsilon(8)=interp1(ITO(1,:),ITO(2,:),wavelength);
epsilon(7)=interp1(pedot(1,:),pedot(2,:),wavelength);
epsilon(6)=interp1(tpd(1,:),tpd(2,:),wavelength);
epsilon(5)=interp1(alq3(1,:),alq3(2,:),wavelength);
epsilon(4)=interp1(bcp(1,:),bcp(2,:),wavelength);
epsilon(3)=interp1(mg(1,:),mg(2,:),wavelength);
epsilon(2)=interp1(ag(1,:),ag(2,:),wavelength);
epsilon(1)=interp1(air(1,:),air(2,:),wavelength);  
N=max(size(epsilon));
res=16; % resolution of Z in each layer (Z is the surface perpendicular coordinate)
% -------------------------------------------------------------------------     

% ------------------ Define the wavevector boundaries ---------------------        
k=2*pi/wavelength/1e-9*sqrt(epsilon);

u_crit=sqrt(1/epsilon(dipole_layer)) % total internal reflection
u_rad=max([1:max(size(u))].*(u_crit>u))
u_crit_glass=sqrt(epsilon(N-1)/epsilon(dipole_layer))
u_glass=max([1:max(size(u))].*(u_crit_glass>u))
% -------------------------------------------------------------------------
 
% ----------- Calculation loop for different dipole positions -------------
for t=[1:max(size(dipole_position_array))],
    dipole_position=dipole_position_array(t); %distance from interface with previous layer)
    z0=[0 cumsum(thickness)];
    z0=z0-(z0(dipole_layer-1)+dipole_position); %set dipole position to z=0

    for n=[1:N],
        h(n,:)=k(dipole_layer)*sqrt(epsilon(n)/epsilon(dipole_layer)-cos(x).^2); % normalized surface perpendicular wavevector
    end

    % CB and FB are c'/c and f'/f respectively , c and f are the eigenfunction coefficients for the Green's functions 
    CB(1,:)=0*x; %from radiation condition
    FB(1,:)=0*x; %from radiation condition

    for n=[2:dipole_layer],
        R_perp(n,:)=(h(n,:)-h(n-1,:))./(h(n,:)+h(n-1,:));
        R_para(n,:)=(epsilon(n)*h(n-1,:)-epsilon(n-1)*h(n,:))./(epsilon(n)*h(n-1,:)+epsilon(n-1)*h(n,:));
        CB(n,:)=exp(-2*i*h(n,:)*z0(n-1)).*( R_perp(n,:)+CB(n-1,:).*exp(2*i*h(n-1,:)*z0(n-1)))./(1+R_perp(n,:).*CB(n-1,:).*exp(2*i*h(n-1,:)*z0(n-1)));
        FB(n,:)=exp(-2*i*h(n,:)*z0(n-1)).*(-R_para(n,:)+FB(n-1,:).*exp(2*i*h(n-1,:)*z0(n-1)))./(1-R_para(n,:).*FB(n-1,:).*exp(2*i*h(n-1,:)*z0(n-1)));
    end
    
    % CT and FT are c/c' and f/f' respectively
    CT(N,:)=0*x; %from radiation condition
    FT(N,:)=0*x; %from radiation condition
    
    for n=[N-1:-1:dipole_layer],
        R_perp(n,:)=(h(n,:)-h(n+1,:))./(h(n,:)+h(n+1,:));
        R_para(n,:)=(epsilon(n)*h(n+1,:)-epsilon(n+1)*h(n,:))./(epsilon(n)*h(n+1,:)+epsilon(n+1)*h(n,:));
        CT(n,:)=exp(2*i*h(n,:)*z0(n)).*( R_perp(n,:)+CT(n+1,:).*exp(-2*i*h(n+1,:)*z0(n)))./(1+R_perp(n,:).*CT(n+1,:).*exp(-2*i*h(n+1,:)*z0(n)));
        FT(n,:)=exp(2*i*h(n,:)*z0(n)).*(-R_para(n,:)+FT(n+1,:).*exp(-2*i*h(n+1,:)*z0(n)))./(1-R_para(n,:).*FT(n+1,:).*exp(-2*i*h(n+1,:)*z0(n)));
    end
    
    c(dipole_layer,:) =(CB(dipole_layer,:)+1).*CT(dipole_layer,:)./(1-CB(dipole_layer,:).*CT(dipole_layer,:));
    cd(dipole_layer,:)=(CT(dipole_layer,:)+1).*CB(dipole_layer,:)./(1-CB(dipole_layer,:).*CT(dipole_layer,:));
    
    f_perp(dipole_layer,:) = (FB(dipole_layer,:)+1).*FT(dipole_layer,:)./(1-FB(dipole_layer,:).*FT(dipole_layer,:));
    fd_perp(dipole_layer,:)= (FT(dipole_layer,:)+1).*FB(dipole_layer,:)./(1-FB(dipole_layer,:).*FT(dipole_layer,:));
    
    f_para(dipole_layer,:) =-(1-FB(dipole_layer,:)).*FT(dipole_layer,:)./(1-FB(dipole_layer,:).*FT(dipole_layer,:));
    fd_para(dipole_layer,:)= (1-FT(dipole_layer,:)).*FB(dipole_layer,:)./(1-FB(dipole_layer,:).*FT(dipole_layer,:));

    for n=[dipole_layer:-1:2],
        c(n-1,:) =0.5*exp( i*h(n-1,:)*z0(n-1)).*(((n==dipole_layer)+c(n,:)).*exp(-i*h(n,:)*z0(n-1)).*(1+h(n,:)./h(n-1,:))+cd(n,:).*exp(i*h(n,:)*z0(n-1)).*(1-h(n,:)./h(n-1,:)));
        cd(n-1,:)=0.5*exp(-i*h(n-1,:)*z0(n-1)).*(((n==dipole_layer)+c(n,:)).*exp(-i*h(n,:)*z0(n-1)).*(1-h(n,:)./h(n-1,:))+cd(n,:).*exp(i*h(n,:)*z0(n-1)).*(1+h(n,:)./h(n-1,:)));
        f_perp(n-1,:) =0.5*exp( i*h(n-1,:)*z0(n-1)).*(((n==dipole_layer)+f_perp(n,:)).*exp(-i*h(n,:)*z0(n-1)).*(k(n)/k(n-1)+k(n-1)/k(n)*h(n,:)./h(n-1,:))+fd_perp(n,:).*exp(i*h(n,:)*z0(n-1)).*(k(n)/k(n-1)-k(n-1)/k(n)*h(n,:)./h(n-1,:)));
        fd_perp(n-1,:)=0.5*exp(-i*h(n-1,:)*z0(n-1)).*(((n==dipole_layer)+f_perp(n,:)).*exp(-i*h(n,:)*z0(n-1)).*(k(n)/k(n-1)-k(n-1)/k(n)*h(n,:)./h(n-1,:))+fd_perp(n,:).*exp(i*h(n,:)*z0(n-1)).*(k(n)/k(n-1)+k(n-1)/k(n)*h(n,:)./h(n-1,:)));
        f_para(n-1,:) =0.5*exp( i*h(n-1,:)*z0(n-1)).*(((n==dipole_layer)+f_para(n,:)).*exp(-i*h(n,:)*z0(n-1)).*(k(n)/k(n-1)+k(n-1)/k(n)*h(n,:)./h(n-1,:))+fd_para(n,:).*exp(i*h(n,:)*z0(n-1)).*(k(n)/k(n-1)-k(n-1)/k(n)*h(n,:)./h(n-1,:)));
        fd_para(n-1,:)=0.5*exp(-i*h(n-1,:)*z0(n-1)).*(((n==dipole_layer)+f_para(n,:)).*exp(-i*h(n,:)*z0(n-1)).*(k(n)/k(n-1)-k(n-1)/k(n)*h(n,:)./h(n-1,:))+fd_para(n,:).*exp(i*h(n,:)*z0(n-1)).*(k(n)/k(n-1)+k(n-1)/k(n)*h(n,:)./h(n-1,:)));
    end
    
    for n=[dipole_layer:N-1],
        c(n+1,:) =0.5*exp( i*h(n+1,:)*z0(n)).*(c(n,:).*exp(-i*h(n,:)*z0(n)).*(1+h(n,:)./h(n+1,:))+((n==dipole_layer)+cd(n,:)).*exp(i*h(n,:)*z0(n)).*(1-h(n,:)./h(n+1,:)));
        cd(n+1,:)=0.5*exp(-i*h(n+1,:)*z0(n)).*(c(n,:).*exp(-i*h(n,:)*z0(n)).*(1-h(n,:)./h(n+1,:))+((n==dipole_layer)+cd(n,:)).*exp(i*h(n,:)*z0(n)).*(1+h(n,:)./h(n+1,:)));
        f_perp(n+1,:) =0.5*exp( i*h(n+1,:)*z0(n)).*(f_perp(n,:).*exp(-i*h(n,:)*z0(n)).*(k(n)/k(n+1)+k(n+1)/k(n)*h(n,:)./h(n+1,:))+(fd_perp(n,:)+(n==dipole_layer)).*exp(i*h(n,:)*z0(n)).*(k(n)/k(n+1)-k(n+1)/k(n)*h(n,:)./h(n+1,:)));
        fd_perp(n+1,:)=0.5*exp(-i*h(n+1,:)*z0(n)).*(f_perp(n,:).*exp(-i*h(n,:)*z0(n)).*(k(n)/k(n+1)-k(n+1)/k(n)*h(n,:)./h(n+1,:))+(fd_perp(n,:)+(n==dipole_layer)).*exp(i*h(n,:)*z0(n)).*(k(n)/k(n+1)+k(n+1)/k(n)*h(n,:)./h(n+1,:)));
        f_para(n+1,:) =0.5*exp( i*h(n+1,:)*z0(n)).*(f_para(n,:).*exp(-i*h(n,:)*z0(n)).*(k(n)/k(n+1)+k(n+1)/k(n)*h(n,:)./h(n+1,:))+(fd_para(n,:)-(n==dipole_layer)).*exp(i*h(n,:)*z0(n)).*(k(n)/k(n+1)-k(n+1)/k(n)*h(n,:)./h(n+1,:)));
        fd_para(n+1,:)=0.5*exp(-i*h(n+1,:)*z0(n)).*(f_para(n,:).*exp(-i*h(n,:)*z0(n)).*(k(n)/k(n+1)-k(n+1)/k(n)*h(n,:)./h(n+1,:))+(fd_para(n,:)-(n==dipole_layer)).*exp(i*h(n,:)*z0(n)).*(k(n)/k(n+1)+k(n+1)/k(n)*h(n,:)./h(n+1,:)));
    end
    
    % b is the inverse lifetime of a dipole
    b_perp(t,:) = real(dx.*(3/2*(f_perp(dipole_layer,:)+fd_perp(dipole_layer,:)).*(cos(x).^3)));
    b_para(t,:) = real(dx.*(3/4*((c(dipole_layer,:)+cd(dipole_layer,:))+(sin(x).^2).*(f_para(dipole_layer,:)-fd_para(dipole_layer,:))).*cos(x))); 

    b_perp_sum(t) = 1-q + q*(1+sum(b_perp(t,:)));
    b_para_sum(t) = 1-q + q*(1+sum(b_para(t,:)));

    % power_perp/para(_u) is the surface perpendicular component of the Poynting vector 
    for n=[1:N-1], % This loop is for the Poynting vector at the interfaces
        power_perp_u(t,:,n) = (real(du.*(-3/4*q*(u.^3)./abs(1-u.^2).*sqrt(epsilon(n)/epsilon(dipole_layer)-u.^2).*conj(sqrt(epsilon(n)))/sqrt(epsilon(n)).*(f_perp(n,:).*exp(-i*h(n,:)*z0(n))-(fd_perp(n,:)+(n==dipole_layer)).*exp(i*h(n,:)*z0(n))).*conj((f_perp(n,:).*exp(-i*h(n,:)*z0(n))+(fd_perp(n,:)+(n==dipole_layer)).*exp(i*h(n,:)*z0(n)))))));
        power_para_u(t,:,n) = (real(du.*(-3/8*q*u.*(conj(sqrt(epsilon(n)/epsilon(dipole_layer)-u.^2))./abs(1-u.^2).*(c(n,:).*exp(-i*h(n,:)*z0(n))+(cd(n,:)+(n==dipole_layer)).*exp(i*h(n,:)*z0(n))).*conj((c(n,:).*exp(-i*h(n,:)*z0(n))-(cd(n,:)+(n==dipole_layer)).*exp(i*h(n,:)*z0(n))))+sqrt(epsilon(n)/epsilon(dipole_layer)-u.^2).*conj(sqrt(epsilon(n)))/sqrt(epsilon(n)).*(f_para(n,:).*exp(-i*h(n,:)*z0(n))-(fd_para(n,:)-(n==dipole_layer)).*exp(i*h(n,:)*z0(n))).*conj((f_para(n,:).*exp(-i*h(n,:)*z0(n))+(fd_para(n,:)-(n==dipole_layer)).*exp(i*h(n,:)*z0(n))))))));
    end

    if t==1,
        for n=1:N-2, % This loop is for the Poynting vector at many points inside the structure
            for jz=1:res
                dZ=thickness/res; % There is no "du" in this loop's Poynting vector calculation
                Z((n-1)*res+jz)=z0(n)+dZ(n)*jz; % for n=1, Z is for second layer
                power_perp_u2((n-1)*res+jz,:,t) = (real((-3/4*q*(u.^3)./abs(1-u.^2).*sqrt(epsilon(n+1)/epsilon(dipole_layer)-u.^2).*conj(sqrt(epsilon(n+1)))/sqrt(epsilon(n+1)).*(f_perp(n+1,:).*exp(-i*h(n+1,:)*Z((n-1)*res+jz))-(fd_perp(n+1,:)+(n+1==dipole_layer)).*exp(i*h(n+1,:)*Z((n-1)*res+jz))).*conj((f_perp(n+1,:).*exp(-i*h(n+1,:)*Z((n-1)*res+jz))+(fd_perp(n+1,:)+(n+1==dipole_layer)).*exp(i*h(n+1,:)*Z((n-1)*res+jz)))))));
                power_para_u2((n-1)*res+jz,:,t) = (real((-3/8*q*u.*(conj(sqrt(epsilon(n+1)/epsilon(dipole_layer)-u.^2))./abs(1-u.^2).*(c(n+1,:).*exp(-i*h(n+1,:)*Z((n-1)*res+jz))+(cd(n+1,:)+(n+1==dipole_layer)).*exp(i*h(n+1,:)*Z((n-1)*res+jz))).*conj((c(n+1,:).*exp(-i*h(n+1,:)*Z((n-1)*res+jz))-(cd(n+1,:)+(n+1==dipole_layer)).*exp(i*h(n+1,:)*Z((n-1)*res+jz))))+sqrt(epsilon(n+1)/epsilon(dipole_layer)-u.^2).*conj(sqrt(epsilon(n+1)))/sqrt(epsilon(n+1)).*(f_para(n+1,:).*exp(-i*h(n+1,:)*Z((n-1)*res+jz))-(fd_para(n+1,:)-(n+1==dipole_layer)).*exp(i*h(n+1,:)*Z((n-1)*res+jz))).*conj((f_para(n+1,:).*exp(-i*h(n+1,:)*Z((n-1)*res+jz))+(fd_para(n+1,:)-(n+1==dipole_layer)).*exp(i*h(n+1,:)*Z((n-1)*res+jz))))))));
            end
        end
        frac_perp_u(:,:,t)=abs(diff([power_perp_u2(:,:,t);zeros(1,size(u,2))])/b_perp_sum); frac_para_u(:,:,t)=abs(diff([power_para_u2(:,:,t);zeros(1,size(u,2))])/b_para_sum);
    end
    
    power_perp=sum(power_perp_u,2); power_para=sum(power_para_u,2);
    frac_perp(t,:)=diff([0 power_perp(t,:) 0]); % absorption in one layer
    frac_para(t,:)=diff([0 power_para(t,:) 0]);

    rad_perp=sum(power_perp_u(:,[1:u_rad],N-1),2); % radiated output
    rad_para=sum(power_para_u(:,[1:u_rad],N-1),2);

end
% -------------------------------------------------------------------------

outcoupling=2/3*rad_para'./b_para_sum+1/3*rad_perp'./b_perp_sum; % Averaged outcoupling
b_total_sum=2/3*b_para_sum+1/3*b_perp_sum; % Averaged inverse lifetime of the dipole
frac_total=2/3*frac_para+1/3*frac_perp; % Averaged absorption in each layer
glass=((abs(frac_total(:,N))+abs(frac_total(:,N-1)))'./b_total_sum)-outcoupling; % Averaged waveguiding in glass

% ------------------------------- FIGURES ---------------------------------
% Figures are published in Celebi, Heidel and Baldo - Optics Express 15,1762-1772 (2007)

hold off

figure(1),clf,
area(1e9*dipole_position_array,[outcoupling; glass; abs(frac_total(:,8)'./b_total_sum)+abs(frac_total(:,7)'./b_total_sum);abs(frac_total(:,6)'./b_total_sum)+abs(frac_total(:,4)'./b_total_sum);abs(frac_total(:,3)'./b_total_sum)+abs(frac_total(:,2)'./b_total_sum)]'),
xlabel('Dipole distance to BCP (nm)'), ylabel('Coupling efficiency'), legend('air','waveguiding','ITO','Organics','Top contact')
title('Averaged Energy transport of the dipole')

figure(2),clf
theta_air=real(acos(sqrt(1-epsilon(dipole_layer)*u([2:u_rad]).^2)));  % Remember k_0 is defined at dipole layer
theta_glass=real(acos(sqrt(1-epsilon(dipole_layer)/epsilon(N)*u([2:u_glass]).^2)));
for t=[1:max(size(dipole_position_array))],
    p_angle_air=(power_perp_u(t,[2:u_rad],N-1).*sqrt(1/epsilon(dipole_layer))./tan(theta_air));
    polar(theta_air,p_angle_air/max(p_angle_air),'r-'),hold on,polar(-theta_air,p_angle_air/max(p_angle_air),'r-')
    p_angle_glass=(power_perp_u(t,[2:u_glass],N-1).*sqrt(epsilon(N)/epsilon(dipole_layer))./tan(theta_glass));
    polar(theta_glass,p_angle_glass/max(p_angle_glass),'b-'),polar(-theta_glass,p_angle_glass/max(p_angle_glass),'b-')
end
polar(theta_air,cos(theta_air),'g-'),polar(-theta_air,cos(theta_air),'g-'),%title('Perpendicular dipole angular emission profile in glass (blue) and air (red) compared to ideal (green)')
title('Averaged Energy transport of the dipole'), hold off
   
figure(3),clf,
theta_air=real(acos(sqrt(1-epsilon(dipole_layer)*u([2:u_rad]).^2)));  % Remember k_0 is defined at dipole layer
theta_glass=real(acos(sqrt(1-epsilon(dipole_layer)/epsilon(N)*u([2:u_glass]).^2)));
for t=[1:max(size(dipole_position_array))],
    p_angle_air=(power_para_u(t,[2:u_rad],N-1).*sqrt(1/epsilon(dipole_layer))./tan(theta_air));
    polar(theta_air,p_angle_air/max(p_angle_air),'r-'),hold on,polar(-theta_air,p_angle_air/max(p_angle_air),'r-')
    p_angle_glass=(power_para_u(t,[2:u_glass],N-1).*sqrt(epsilon(N)/epsilon(dipole_layer))./tan(theta_glass));
    polar(theta_glass,p_angle_glass/max(p_angle_glass),'b-'),polar(-theta_glass,p_angle_glass/max(p_angle_glass),'b-')
end
polar(theta_air,cos(theta_air),'g-'),polar(-theta_air,cos(theta_air),'g-'),%title('Parallel dipole angular emission profile in glass (blue) and air (red) compared to ideal (green)')
title('Averaged Energy transport of the dipole'), hold off

% ------------------- Mode distribution figures ---------------------------
u_cond=[0:max(u)/250:max(u)]; % Smoothed arrays for display purposes
frac_perp_cond=interp2(u,Z(1:size(Z,2)),frac_perp_u(:,:,1),u_cond,Z(1:size(Z,2))'); 
frac_para_cond=interp2(u,Z(1:size(Z,2)),frac_para_u(:,:,1),u_cond,Z(1:size(Z,2))');
frac_iso_cond=2/3*frac_para_cond+1/3*frac_perp_cond;

x1=0; x2=1.5; y1=-150; y2=250;
figure(4),clf
surf(u_cond,[1e9*Z(1:3*res-1)],log10([abs(frac_perp_cond(1:3*res-1,:,1))])),view(2),caxis([-4 0]),colorbar
hold on
surf(u_cond,[1e9*Z(4*res+1:(N-2)*res-1)],log10([abs(frac_perp_cond(4*res+1:(N-2)*res-1,:,1))])),view(2),caxis([-4 0]),colorbar
grid off, axis([x1 x2 y1 y2]), colormap hot
xlabel('k_x/k_0'),ylabel('Distance from the dipole (nm)'), title('Perpendicular dipole mode distribution')

figure(5),clf
surf(u_cond,[1e9*Z(1:3*res-1)],log10([(abs(frac_para_cond(1:3*res-1,:,1)))])),view(2),caxis([-4 0]),colorbar
hold on
surf(u_cond,[1e9*Z(4*res+1:(N-2)*res-1)],log10([(abs(frac_para_cond(4*res+1:(N-2)*res-1,:,1)))])),view(2),caxis([-4 0]),colorbar
% xlabel('k_x/k_0'),ylabel('Distance from the dipole (nm)'),title('Para')
grid off, axis([x1 x2 y1 y2]), colormap hot
xlabel('k_x/k_0'),ylabel('Distance from the dipole (nm)'), title('Parallel dipole mode distribution')
% -------------------------------------------------------------------------

% OPTIONAL FIGURES:

% figure(6),clf,
% area(1e9*dipole_position_array,[abs(frac_perp(:,2)'./b_perp_sum);abs(frac_perp(:,3)'./b_perp_sum); abs(frac_perp(:,4)'./b_perp_sum);abs(frac_perp(:,6)'./b_perp_sum);abs(frac_perp(:,7)'./b_perp_sum);abs(frac_perp(:,8)'./b_perp_sum);abs(frac_perp(:,9)'./b_perp_sum);abs(frac_perp(:,10)'./b_perp_sum)]'),
% title('Perpendicular Dipole energy Transfer'),xlabel('Dipole distance to HTL (nm)'), ylabel('Coupling efficiency'), legend('ag','mg','bcp','tpd','pedot','ITO','glass','AIR')
% hold on,
% plot(1e9*dipole_position_array,1-rad_perp'./b_perp_sum)
% title('Energy transport for the perpendicular dipole'), hold off

% figure(7),clf, 
% area(1e9*dipole_position_array,[abs(frac_para(:,2)'./b_para_sum);abs(frac_para(:,3)'./b_para_sum); abs(frac_para(:,4)'./b_para_sum);abs(frac_para(:,6)'./b_para_sum);abs(frac_para(:,7)'./b_para_sum);abs(frac_para(:,8)'./b_para_sum);abs(frac_para(:,9)'./b_para_sum);abs(frac_para(:,10)'./b_para_sum)]'),
% title('Parallel Dipole energy Transfer'),xlabel('Dipole distance to HTL (nm)'), ylabel('Coupling efficiency'), legend('ag','mg','bcp','tpd','pedot','ITO','glass','AIR')
% hold on,
% plot(1e9*dipole_position_array,1-rad_para'./b_para_sum)
% title('Energy transport for the parallel dipole'), hold off

% figure(8),clf,
% theta_air=real(acos(sqrt(1-epsilon(dipole_layer)*u([2:u_rad]).^2)));  % Remember k_0 is defined at dipole layer
% theta_glass=real(acos(sqrt(1-epsilon(dipole_layer)/epsilon(N)*u([2:u_glass]).^2)));
% for t=[1:max(size(dipole_position_array))],
%     p_angle_air=2/3*(power_para_u(t,[2:u_rad],N-1).*sqrt(1/epsilon(dipole_layer))./tan(theta_air))+1/3*(power_perp_u(t,[2:u_rad],N-1).*sqrt(1/epsilon(dipole_layer))./tan(theta_air));
%     polar(theta_air,p_angle_air/max(p_angle_air),'r-'),hold on,polar(-theta_air,p_angle_air/max(p_angle_air),'r-')
%     p_angle_glass=2/3*(power_para_u(t,[2:u_glass],N-1).*sqrt(epsilon(N)/epsilon(dipole_layer))./tan(theta_glass))+1/3*(power_perp_u(t,[2:u_glass],N-1).*sqrt(epsilon(N)/epsilon(dipole_layer))./tan(theta_glass));
%     polar(theta_glass,p_angle_glass/max(p_angle_glass),'b-'),polar(-theta_glass,p_angle_glass/max(p_angle_glass),'b-')
% end
% polar(theta_air,cos(theta_air),'g-'),polar(-theta_air,cos(theta_air),'g-'),%title('Average angular emission profile in glass (blue) and air (red) compared to ideal (green)')
% title('Averaged angular emission profile for the dipole'), hold off
% -------------------------------------------------------------------------

% ------------------------ END OF THE PROGRAM -----------------------------